@@ -14,7 +14,8 @@ setClass("response",
 		fixed="logical",
 		y = "matrix",
 		x = "matrix",
-		npar = "numeric" # this is not really needed as it simply is length(unlist(parameters))
+		npar = "numeric", # this is not really needed as it simply is length(unlist(parameters))
+		constr = "ANY"
 	)
 )
 
@@ -34,5 +35,3 @@ setMethod("getdf","response",
 	}
 )
 
-
-
